@@ -17,7 +17,7 @@ from traitlets import (
     Undefined)
 from ipython_genutils.py3compat import string_types, PY3
 from IPython.display import display
-from json import dumps as jsondumps
+from json import loads as jsonloads, dumps as jsondumps
 
 from base64 import standard_b64decode, standard_b64encode
 
@@ -158,14 +158,6 @@ def _buffer_list_equal(a, b):
                 return False
     return True
 
-def _json_equal(a, b):
-    """Compare the JSON strings generated by two objects."""
-    # Comparing the JSON strings automatically takes into account the automatic
-    # Python conversions that happen, such as tuple to list, when encoding to
-    # JSON. We try to make the strings deterministic and small with the options
-    # we use.
-    return (jsondumps(a, separators=(',',':'), sort_keys=True, allow_nan=False)
-            == jsondumps(b, separators=(',',':'), sort_keys=True, allow_nan=False))
 
 class LoggingHasTraits(HasTraits):
     """A parent class for HasTraits that log.
@@ -647,8 +639,10 @@ class Widget(LoggingHasTraits):
             # model_state, buffer_paths, buffers
             split_value = _remove_buffers({ key: to_json(value, self)})
             split_lock = _remove_buffers({ key: self._property_lock[key]})
-            # Compare state and buffer_paths
-            if (_json_equal(split_value[0], split_lock[0])
+            # A roundtrip conversion through json in the comparison takes care of
+            # idiosyncracies of how python data structures map to json, for example
+            # tuples get converted to lists.
+            if (jsonloads(jsondumps(split_value[0])) == split_lock[0]
                 and split_value[1] == split_lock[1]
                 and _buffer_list_equal(split_value[2], split_lock[2])):
                 return False
@@ -743,7 +737,7 @@ class Widget(LoggingHasTraits):
                 continue
             elif (isinstance(trait, (Container, Dict)) and
                   trait.default_value == Undefined and
-                  len(value) == 0):
+                  (value is None or len(value) == 0)):
                 # Empty container, and dynamic default will be empty
                 continue
             yield key
@@ -763,14 +757,14 @@ class Widget(LoggingHasTraits):
 _FALLBACK_HTML_TEMPLATE = """\
 <p>Failed to display Jupyter Widget of type <code>{widget_type}</code>.</p>
 <p>
-  If you're reading this message in Jupyter Notebook or JupyterLab, it may mean
+  If you're reading this message in the Jupyter Notebook or JupyterLab Notebook, it may mean
   that the widgets JavaScript is still loading. If this message persists, it
   likely means that the widgets JavaScript library is either not installed or
   not enabled. See the <a href="https://ipywidgets.readthedocs.io/en/stable/user_install.html">Jupyter
   Widgets Documentation</a> for setup instructions.
 </p>
 <p>
-  If you're reading this message in another notebook frontend (for example, a static
+  If you're reading this message in another frontend (for example, a static
   rendering on GitHub or <a href="https://nbviewer.jupyter.org/">NBViewer</a>),
   it may mean that your frontend doesn't currently support widgets.
 </p>
